@@ -19,14 +19,23 @@
 # SPDX-FileCopyrightText: 2023 Shun Sakai <sorairolake@protonmail.ch>
 # SPDX-FileCopyrightText: 2024 Rivos Inc.
 # SPDX-FileCopyrightText: 2024 Anthony Loiseau <anthony.loiseau@allcircuits.com>
+# SPDX-FileCopyrightText: 2024 Yongmin Hong <yewon@revi.email>
 # SPDX-FileCopyrightText: 2025 Raphael Schlarb <info@raphael.schlarb.one>
 # SPDX-FileCopyrightText: 2025 András Nagy <nagyandris0718@gmail.com>
 # SPDX-FileCopyrightText: 2025 Kiko Fernandez-Reyes <kiko@erlang.org>
 # SPDX-FileCopyrightText: 2025 Hatzka <hatzka@nezumi.studio>
 # SPDX-FileCopyrightText: 2025 Johannes HAMPP <johannes.hampp@openenergytransition.org>
+# SPDX-FileCopyrightText: 2025 Simon Barth <simon.barth@gmx.de>
+# SPDX-FileCopyrightText: 2025 Maximilian Franzke <mfr@nzke.net>
+# SPDX-FileCopyrightText: 2025 Sacha-Élie Ayoun <sachaayoun@gmail.com>
+# SPDX-FileCopyrightText: 2025 Thomas Gilon <thomas.gilon@openenergytransition.org>
+# SPDX-FileCopyrightText: 2025 Manlio Perillo <manlio.perillo@gmail.com>
+# SPDX-FileCopyrightText: 2025 Matthias Schoettle <opensource@mattsch.com>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
+# pylint: disable=too-many-lines
+
 """Module for parsing and creating comments. Just enough to deal with comment
 headers, in any case.
 """
@@ -45,8 +54,8 @@ _LOGGER = logging.getLogger(__name__)
 
 
 class MultiLineSegments(NamedTuple):
-    """Components that make up a multi-line comment style, e.g. '/*', '*', and
-    '*/'.
+    """Components that make up a multi-line comment style, e.g. ``'/*'``,
+    ``'*'``, and ``'*/'``.
     """
 
     start: str
@@ -821,6 +830,7 @@ EXTENSION_COMMENT_STYLE_MAP = {
     # Visual Studio solution file, officially uncommentable:
     ".sln": UncommentableCommentStyle,
     ".sls": LispCommentStyle,  # Scheme Library Source (R6RS)
+    ".smk": PythonCommentStyle,
     ".sml": MlCommentStyle,
     ".soy": CppCommentStyle,
     ".sps": LispCommentStyle,  # Scheme Program Source (R6RS)
@@ -877,6 +887,9 @@ EXTENSION_COMMENT_STYLE_MAP_LOWERCASE = {
 }
 
 FILENAME_COMMENT_STYLE_MAP = {
+    ".arcconfig": UncommentableCommentStyle,  # is a JSON file
+    ".arclint": UncommentableCommentStyle,  # is a JSON file
+    ".arcunit": UncommentableCommentStyle,  # is a JSON file
     ".bashrc": PythonCommentStyle,
     ".bazelignore": PythonCommentStyle,
     ".bazelrc": PythonCommentStyle,
@@ -900,6 +913,7 @@ FILENAME_COMMENT_STYLE_MAP = {
     ".mdlrc": PythonCommentStyle,  # Markdown-linter config
     ".npmignore": PythonCommentStyle,
     ".npmrc": SemicolonCommentStyle,
+    ".nvmrc": PythonCommentStyle,
     ".prettierrc": UncommentableCommentStyle,  # could either be JSON or YAML
     ".prettierignore": PythonCommentStyle,
     ".pylintrc": PythonCommentStyle,
@@ -909,6 +923,7 @@ FILENAME_COMMENT_STYLE_MAP = {
     ".taprc": PythonCommentStyle,
     ".vimrc": VimCommentStyle,
     ".zshrc": PythonCommentStyle,
+    ".yamllint": PythonCommentStyle,  # is a YAML file
     ".yarnrc": PythonCommentStyle,
     "ansible.cfg": PythonCommentStyle,
     "archive.sctxar": UncommentableCommentStyle,  # SuperCollider global archive
@@ -920,6 +935,9 @@ FILENAME_COMMENT_STYLE_MAP = {
     "Containerfile": PythonCommentStyle,
     "Dockerfile": PythonCommentStyle,
     "Doxyfile": PythonCommentStyle,
+    "dune": SemicolonCommentStyle,  # OCaml build system
+    "dune-project": SemicolonCommentStyle,  # OCaml build system
+    "dune-workspace": SemicolonCommentStyle,  # OCaml build system
     "Earthfile": PythonCommentStyle,
     "flake.lock": UncommentableCommentStyle,  # is a JSON file
     "Gemfile": PythonCommentStyle,
@@ -931,6 +949,7 @@ FILENAME_COMMENT_STYLE_MAP = {
     "Makefile": PythonCommentStyle,
     "MANIFEST.in": PythonCommentStyle,
     "manifest": PythonCommentStyle,  # used by cdist
+    "matplotlibrc": PythonCommentStyle,
     "meson.build": PythonCommentStyle,
     "meson_options.txt": PythonCommentStyle,
     "poetry.lock": UncommentableCommentStyle,
@@ -941,6 +960,8 @@ FILENAME_COMMENT_STYLE_MAP = {
     "requirements.txt": PythonCommentStyle,
     "ROOT": MlCommentStyle,
     "setup.cfg": PythonCommentStyle,
+    "Snakefile": PythonCommentStyle,
+    "uv.lock": UncommentableCommentStyle,
     "yarn.lock": UncommentableCommentStyle,
 }
 
